@@ -1,18 +1,23 @@
 from collections import defaultdict
 
 import graphblas as gb
-from graphblas import Matrix, Vector, binary, replace, select, unary
+from graphblas import Matrix, binary, replace, select, unary
 
 import graphblas_algorithms as ga
 
 from . import _utils
 from ._caching import get_reduce_to_scalar, get_reduce_to_vector
-from .graph import Graph
-
-
-def get_A(G, mask=None):
-    """A"""
-    return G._A
+from .graph import (
+    Graph,
+    get_A,
+    get_diag,
+    get_iso_value,
+    get_offdiag,
+    has_negative_diagonal,
+    has_negative_edgesm,
+    has_negative_edgesp,
+    is_iso,
+)
 
 
 def get_AT(G, mask=None):
@@ -24,22 +29,6 @@ def get_AT(G, mask=None):
     return cache["AT"]
 
 
-def get_offdiag(G, mask=None):
-    """select.offdiag(A)"""
-    A = G._A
-    cache = G._cache
-    if "offdiag" not in cache:
-        if cache.get("has_self_edges") is False:
-            cache["offdiag"] = A
-        else:
-            cache["offdiag"] = select.offdiag(A).new(name="offdiag")
-    if "has_self_edges" not in cache:
-        cache["has_self_edges"] = A.nvals > cache["offdiag"].nvals
-    if not cache["has_self_edges"]:
-        cache["offdiag"] = A
-    return cache["offdiag"]
-
-
 def get_Up(G, mask=None):
     """select.triu(A)"""
     A = G._A
@@ -126,48 +115,24 @@ def get_Lm(G, mask=None):
     return cache["L-"]
 
 
-def get_diag(G, mask=None):
-    """select.diag(A)"""
-    A = G._A
-    cache = G._cache
-    if "diag" not in cache:
-        if cache.get("has_self_edges") is False:
-            cache["diag"] = Vector(A.dtype, size=A.nrows, name="diag")
-        elif "U+" in cache:
-            cache["diag"] = cache["U+"].diag(name="diag")
-        elif "L+" in cache:
-            cache["diag"] = cache["L+"].diag(name="diag")
-        else:
-            cache["diag"] = A.diag(name="diag")
-    if "has_self_edges" not in cache:
-        cache["has_self_edges"] = cache["diag"].nvals > 0
-    if mask is not None:
-        return cache["diag"].dup(mask=mask)
-    return cache["diag"]
-
-
 def get_recip_degreesp(G, mask=None):
     """pair(A & A.T).reduce_rowwise()"""
     A = G._A
     cache = G._cache
-    if "AT" in cache:
-        AT = cache["AT"]
-    else:
-        AT = A.T
+    AT = cache.get("AT", A.T)
     if mask is not None:
         if "recip_degrees+" in cache:
             return cache["recip_degrees+"].dup(mask=mask)
-        elif cache.get("has_self_edges") is False and "recip_degrees-" in cache:
+        if cache.get("has_self_edges") is False and "recip_degrees-" in cache:
             cache["recip_degrees+"] = cache["recip_degrees-"]
             return cache["recip_degrees-"].dup(mask=mask)
-        elif "recip_degrees-" in cache and "diag" in cache:
+        if "recip_degrees-" in cache and "diag" in cache:
             return (unary.one(cache["diag"]) + cache["recip_degrees-"]).new(
                 mask=mask, name="recip_degrees+"
             )
-        elif "recip_degrees-" in cache and not G.get_property("has_self_edges"):
+        if "recip_degrees-" in cache and not G.get_property("has_self_edges"):
             return cache["recip_degrees-"].dup(mask=mask)
-        else:
-            return binary.pair(A & AT).reduce_rowwise().new(mask=mask, name="recip_degrees+")
+        return binary.pair(A & AT).reduce_rowwise().new(mask=mask, name="recip_degrees+")
     if "recip_degrees+" not in cache:
         if cache.get("has_self_edges") is False and "recip_degrees-" in cache:
             cache["recip_degrees+"] = cache["recip_degrees-"]
@@ -205,34 +170,33 @@ def get_recip_degreesm(G, mask=None):
     if mask is not None:
         if "recip_degrees-" in cache:
             return cache["recip_degrees-"].dup(mask=mask)
-        elif cache.get("has_self_edges") is False and "recip_degrees+" in cache:
+        if cache.get("has_self_edges") is False and "recip_degrees+" in cache:
             cache["recip_degrees-"] = cache["recip_degrees+"]
             return cache["recip_degrees-"].dup(mask=mask)
-        elif "recip_degrees+" in cache and "diag" in cache:
+        if "recip_degrees+" in cache and "diag" in cache:
             rv = binary.minus(cache["recip_degrees+"] | unary.one(cache["diag"])).new(
                 mask=mask, name="recip_degrees-"
             )
             rv(rv.V, replace) << rv  # drop 0s
             return rv
-        elif not G.get_property("has_self_edges"):
+        if not G.get_property("has_self_edges"):
             return G.get_property("recip_degrees+", mask=mask)
-        elif "offdiag" in cache:
+        if "offdiag" in cache:
             return (
                 binary.pair(cache["offdiag"] & AT)
                 .reduce_rowwise()
                 .new(mask=mask, name="recip_degrees-")
             )
-        elif "L-" in cache and "U-" in cache:
+        if "L-" in cache and "U-" in cache:
             return (
                 binary.pair(cache["L-"] & AT).reduce_rowwise().new(mask=mask)
                 + binary.pair(cache["U-"] & AT).reduce_rowwise().new(mask=mask)
             ).new(name="recip_degrees-")
-        else:
-            diag = G.get_property("diag", mask=mask)
-            overlap = binary.pair(A & AT).reduce_rowwise().new(mask=mask)
-            rv = binary.minus(overlap | unary.one(diag)).new(name="recip_degrees-")
-            rv(rv.V, replace) << rv  # drop 0s
-            return rv
+        diag = G.get_property("diag", mask=mask)
+        overlap = binary.pair(A & AT).reduce_rowwise().new(mask=mask)
+        rv = binary.minus(overlap | unary.one(diag)).new(name="recip_degrees-")
+        rv(rv.V, replace) << rv  # drop 0s
+        return rv
     if "recip_degrees-" not in cache:
         if cache.get("has_self_edges") is False and "recip_degrees+" in cache:
             cache["recip_degrees-"] = cache["recip_degrees+"]
@@ -276,14 +240,12 @@ def get_total_degreesp(G, mask=None):
     if mask is not None:
         if "total_degrees+" in cache:
             return cache["total_degrees+"].dup(mask=mask)
-        elif cache.get("has_self_edges") is False and "total_degrees-" in cache:
+        if cache.get("has_self_edges") is False and "total_degrees-" in cache:
             cache["total_degrees+"] = cache["total_degrees-"]
             return cache["total_degrees+"].dup(mask=mask)
-        else:
-            return (
-                G.get_property("row_degrees+", mask=mask)
-                + G.get_property("column_degrees+", mask=mask)
-            ).new(name="total_degrees+")
+        return (
+            G.get_property("row_degrees+", mask=mask) + G.get_property("column_degrees+", mask=mask)
+        ).new(name="total_degrees+")
     if "total_degrees+" not in cache:
         if cache.get("has_self_edges") is False and "total_degrees-" in cache:
             cache["total_degrees+"] = cache["total_degrees-"]
@@ -308,14 +270,12 @@ def get_total_degreesm(G, mask=None):
     if mask is not None:
         if "total_degrees-" in cache:
             return cache["total_degrees-"].dup(mask=mask)
-        elif cache.get("has_self_edges") is False and "total_degrees+" in cache:
+        if cache.get("has_self_edges") is False and "total_degrees+" in cache:
             cache["total_degrees-"] = cache["total_degrees+"]
             return cache["total_degrees-"].dup(mask=mask)
-        else:
-            return (
-                G.get_property("row_degrees-", mask=mask)
-                + G.get_property("column_degrees-", mask=mask)
-            ).new(name="total_degrees-")
+        return (
+            G.get_property("row_degrees-", mask=mask) + G.get_property("column_degrees-", mask=mask)
+        ).new(name="total_degrees-")
     if "total_degrees-" not in cache:
         if cache.get("has_self_edges") is False and "total_degrees+" in cache:
             cache["total_degrees-"] = cache["total_degrees+"]
@@ -344,10 +304,7 @@ def get_total_recipp(G, mask=None):
         elif "recip_degrees+" in cache:
             cache["total_recip+"] = cache["recip_degrees+"].reduce().get(0)
         else:
-            if "AT" in cache:
-                AT = cache["AT"]
-            else:
-                AT = A.T
+            AT = cache.get("AT", A.T)
             cache["total_recip+"] = binary.pair(A & AT).reduce_scalar().get(0)
     if "has_self_edges" not in cache and "total_recip-" in cache:
         cache["has_self_edges"] = cache["total_recip+"] > cache["total_recip-"]
@@ -388,6 +345,8 @@ def has_self_edges(G, mask=None):
                 cache["has_self_edges"] = cache["U-"].nvals < cache["U+"].nvals
             else:
                 cache["has_self_edges"] = cache["U-"].nvals + cache["L-"].nvals < A.nvals
+        elif cache.get("has_negative_diagonal") is True:
+            cache["has_self_edges"] = True
         elif "total_recip-" in cache and "total_recip+" in cache:
             cache["has_self_edges"] = cache["total_recip+"] > cache["total_recip-"]
         elif "row_degrees-" in cache and "row_degrees+" in cache:
@@ -427,8 +386,7 @@ def to_directed_graph(G, weight=None, dtype=None):
             return DiGraph.from_networkx(G, weight=weight, dtype=dtype)
     except ImportError:
         pass
-
-    raise TypeError()
+    raise TypeError
 
 
 def to_graph(G, weight=None, dtype=None):
@@ -449,8 +407,7 @@ def to_graph(G, weight=None, dtype=None):
             return ga.Graph.from_networkx(G, weight=weight, dtype=dtype)
     except ImportError:
         pass
-
-    raise TypeError()
+    raise TypeError
 
 
 class AutoDict(dict):
@@ -473,9 +430,14 @@ class AutoDict(dict):
                 get_reduction = get_reduce_to_scalar(key, opname)
             else:
                 get_reduction = get_reduce_to_vector(key, opname, methodname)
-            self[key] = get_reduction
-            return get_reduction
-        raise KeyError(key)
+        elif key.endswith("_diagonal"):
+            # e.g., min_diagonal
+            opname = key[: -len("_diagonal")]
+            get_reduction = get_reduce_to_scalar(key, opname)
+        else:
+            raise KeyError(key)
+        self[key] = get_reduction
+        return get_reduction
 
 
 class DiGraph(Graph):
@@ -507,6 +469,12 @@ class DiGraph(Graph):
                     "total_degrees-",
                     "total_recip+",  # scalar; I don't like this name
                     "total_recip-",  # scalar; I don't like this name
+                    "min_diagonal",
+                    "min_element+",
+                    "min_element-",
+                    "has_negative_diagonal",
+                    "has_negative_edges-",
+                    "has_negative_edges+",
                     "has_self_edges",
                 ]
             )
@@ -528,6 +496,11 @@ class DiGraph(Graph):
             "total_degrees-": get_total_degreesm,
             "total_recip+": get_total_recipp,
             "total_recip-": get_total_recipm,
+            "is_iso": is_iso,
+            "iso_value": get_iso_value,
+            "has_negative_diagonal": has_negative_diagonal,
+            "has_negative_edges-": has_negative_edgesm,
+            "has_negative_edges+": has_negative_edgesp,
             "has_self_edges": has_self_edges,
         }
     )
@@ -568,7 +541,10 @@ class DiGraph(Graph):
     list_to_vector = _utils.list_to_vector
     list_to_mask = _utils.list_to_mask
     list_to_ids = _utils.list_to_ids
+    list_to_keys = _utils.list_to_keys
     matrix_to_dicts = _utils.matrix_to_dicts
+    matrix_to_nodenodemap = _utils.matrix_to_nodenodemap
+    matrix_to_vectornodemap = _utils.matrix_to_vectornodemap
     set_to_vector = _utils.set_to_vector
     to_networkx = _utils.to_networkx
     vector_to_dict = _utils.vector_to_dict
