@@ -55,7 +55,17 @@ class Dispatcher:
     is_k_regular = nxapi.regular.is_k_regular
     is_regular = nxapi.regular.is_regular
     # Shortest Paths
+    floyd_warshall = nxapi.shortest_paths.dense.floyd_warshall
+    floyd_warshall_predecessor_and_distance = (
+        nxapi.shortest_paths.dense.floyd_warshall_predecessor_and_distance
+    )
     has_path = nxapi.shortest_paths.generic.has_path
+    all_pairs_bellman_ford_path_length = (
+        nxapi.shortest_paths.weighted.all_pairs_bellman_ford_path_length
+    )
+    single_source_bellman_ford_path_length = (
+        nxapi.shortest_paths.weighted.single_source_bellman_ford_path_length
+    )
     # Simple Paths
     is_simple_path = nxapi.simple_paths.is_simple_path
     # S Metric
@@ -99,14 +109,26 @@ class Dispatcher:
             import pytest
         except ImportError:  # pragma: no cover (import)
             return
-        skip = [
-            ("test_attributes", {"TestBoruvka", "test_mst.py"}),
-            ("test_weight_attribute", {"TestBoruvka", "test_mst.py"}),
-        ]
+
+        def key(testpath):
+            filename, path = testpath.split(":")
+            classname, testname = path.split(".")
+            return (testname, frozenset({classname, filename}))
+
+        # Reasons to skip tests
+        multi_attributed = "unable to handle multi-attributed graphs"
+        multidigraph = "unable to handle MultiDiGraph"
+        multigraph = "unable to handle MultiGraph"
+
+        # Which tests to skip
+        skip = {
+            key("test_mst.py:TestBoruvka.test_attributes"): multi_attributed,
+            key("test_mst.py:TestBoruvka.test_weight_attribute"): multi_attributed,
+            key("test_dense.py:TestFloyd.test_zero_weight"): multidigraph,
+            key("test_weighted.py:TestBellmanFordAndGoldbergRadzik.test_multigraph"): multigraph,
+        }
         for item in items:
             kset = set(item.keywords)
-            for test_name, keywords in skip:
+            for (test_name, keywords), reason in skip.items():
                 if item.name == test_name and keywords.issubset(kset):
-                    item.add_marker(
-                        pytest.mark.xfail(reason="unable to handle multi-attributed graphs")
-                    )
+                    item.add_marker(pytest.mark.xfail(reason=reason))
