@@ -3,9 +3,10 @@ from graphblas import monoid
 from graphblas_algorithms import algorithms
 from graphblas_algorithms.classes.digraph import to_graph
 from graphblas_algorithms.classes.graph import to_undirected_graph
-from graphblas_algorithms.classes.nodemap import VectorNodeMap
 from graphblas_algorithms.utils import not_implemented_for
 
+from ._utils import normalize_chunksize, partition
+
 __all__ = [
     "triangles",
     "transitivity",
@@ -25,7 +26,7 @@ def triangles(G, nodes=None):
         return algorithms.single_triangle(G, nodes)
     mask = G.list_to_mask(nodes)
     result = algorithms.triangles(G, mask=mask)
-    return G.vector_to_nodemap(result, mask=mask, fillvalue=0)
+    return G.vector_to_nodemap(result, mask=mask, fill_value=0)
 
 
 def transitivity(G):
@@ -47,20 +48,19 @@ def clustering(G, nodes=None, weight=None):
     if nodes in G:
         if G.is_directed():
             return algorithms.single_clustering_directed(G, nodes, weighted=weighted)
-        else:
-            return algorithms.single_clustering(G, nodes, weighted=weighted)
+        return algorithms.single_clustering(G, nodes, weighted=weighted)
     mask = G.list_to_mask(nodes)
     if G.is_directed():
         result = algorithms.clustering_directed(G, weighted=weighted, mask=mask)
     else:
         result = algorithms.clustering(G, weighted=weighted, mask=mask)
-    return G.vector_to_nodemap(result, mask=mask, fillvalue=0.0)
+    return G.vector_to_nodemap(result, mask=mask, fill_value=0.0)
 
 
 def average_clustering(G, nodes=None, weight=None, count_zeros=True):
     G = to_graph(G, weight=weight)  # to directed or undirected
     if len(G) == 0:
-        raise ZeroDivisionError()
+        raise ZeroDivisionError
     weighted = weight is not None
     mask = G.list_to_mask(nodes)
     if G.is_directed():
@@ -75,8 +75,7 @@ def average_clustering(G, nodes=None, weight=None, count_zeros=True):
             weighted=weighted,
             count_zeros=count_zeros,
         )
-    else:
-        return func(G, weighted=weighted, count_zeros=count_zeros, mask=mask)
+    return func(G, weighted=weighted, count_zeros=count_zeros, mask=mask)
 
 
 def _split(L, k):
@@ -93,11 +92,11 @@ def _split(L, k):
 
 
 # TODO: should this move into algorithms?
-def _square_clustering_split(G, node_ids=None, *, nsplits):
+def _square_clustering_split(G, node_ids=None, *, chunksize):
     if node_ids is None:
-        node_ids = G._A.reduce_rowwise(monoid.any).to_coo()[0]
+        node_ids, _ = G._A.reduce_rowwise(monoid.any).to_coo(values=False)
     result = None
-    for chunk_ids in _split(node_ids, nsplits):
+    for chunk_ids in partition(chunksize, node_ids):
         res = algorithms.square_clustering(G, chunk_ids)
         if result is None:
             result = res
@@ -106,27 +105,33 @@ def _square_clustering_split(G, node_ids=None, *, nsplits):
     return result
 
 
-def square_clustering(G, nodes=None, *, nsplits=None):
+def square_clustering(G, nodes=None, *, chunksize="256 MiB"):
+    # `chunksize` is used to split the computation into chunks.
+    # square_clustering computes `A @ A`, which can get very large, even dense.
+    # The default `chunksize` is to choose the number so that `Asubset @ A`
+    # will be about 256 MB if dense.
     G = to_undirected_graph(G)
     if len(G) == 0:
         return {}
-    elif nodes is None:
+
+    chunksize = normalize_chunksize(chunksize, len(G) * G._A.dtype.np_type.itemsize, len(G))
+
+    if nodes is None:
         # Should we use this one for subsets of nodes as well?
-        if nsplits is None:
+        if chunksize is None:
             result = algorithms.square_clustering(G)
         else:
-            result = _square_clustering_split(G, nsplits=nsplits)
-        return G.vector_to_nodemap(result, fillvalue=0)
-    elif nodes in G:
+            result = _square_clustering_split(G, chunksize=chunksize)
+        return G.vector_to_nodemap(result, fill_value=0)
+    if nodes in G:
         idx = G._key_to_id[nodes]
         return algorithms.single_square_clustering(G, idx)
+    ids = G.list_to_ids(nodes)
+    if chunksize is None:
+        result = algorithms.square_clustering(G, ids)
     else:
-        ids = G.list_to_ids(nodes)
-        if nsplits is None:
-            result = algorithms.square_clustering(G, ids)
-        else:
-            result = _square_clustering_split(G, ids, nsplits=nsplits)
-        return G.vector_to_nodemap(result)
+        result = _square_clustering_split(G, ids, chunksize=chunksize)
+    return G.vector_to_nodemap(result)
 
 
 @not_implemented_for("directed")
@@ -139,6 +144,4 @@ def generalized_degree(G, nodes=None):
         return G.vector_to_nodemap(result)
     mask = G.list_to_mask(nodes)
     result = algorithms.generalized_degree(G, mask=mask)
-    rv = VectorNodeMap(result, key_to_id=G._key_to_id)
-    rv._id_to_key = G._id_to_key
-    return rv
+    return G.matrix_to_vectornodemap(result)
