@@ -4,7 +4,7 @@ from graphblas_algorithms import algorithms
 from graphblas_algorithms.classes.digraph import to_directed_graph
 from graphblas_algorithms.utils import not_implemented_for
 
-from .simple_paths import is_simple_path as is_path  # noqa
+from .simple_paths import is_simple_path as is_path  # noqa: F401
 
 __all__ = ["is_tournament", "score_sequence", "tournament_matrix"]
 
